@@ -32,14 +32,11 @@ typedef struct _GnomeScanResolutionSelectorPrivate		GnomeScanResolutionSelectorP
 struct _GnomeScanResolutionSelectorPrivate {
   gboolean		dispose_has_run;
   GnomeScanContext	*context;
-  GtkAdjustment		*x;
-  GtkWidget		*xscale;
-  GtkWidget		*xspin;
-  GtkAdjustment		*y;
-  GtkWidget		*yscale;
-  GtkWidget		*yspin;
-  GtkWidget		*chain;
-  gboolean		chained;
+  GtkAdjustment		*adjustment;
+  GtkWidget		*scale;
+  GtkWidget		*spin;
+  GtkWidget		*combo;
+  GtkListStore		*liststore;
 };
 
 /********************************
@@ -53,7 +50,10 @@ void				gsrs_scanner_selected				(GnomeScanContext *context,
 gchar*				gsrs_format_value				(GtkScale *scale,
 										 gdouble value);
 
-void				gsrs_value_changed				(GtkRange *range,
+void				gsrs_range_value_changed			(GtkRange *range,
+										 GnomeScanResolutionSelector *gsrs);
+
+void 				gsrs_combo_value_changed			(GtkComboBox *combo,
 										 GnomeScanResolutionSelector *gsrs);
 
 void				gsrs_chain_clicked				(GtkButton *button,
@@ -92,6 +92,13 @@ enum {
   N_SIGNALS
 };
 
+enum
+{
+        ENUM_LABEL,
+        ENUM_VALUE,
+        ENUM_N_COLUMNS
+};
+
 static guint signals[N_SIGNALS];
 
 
@@ -130,14 +137,10 @@ gnome_scan_resolution_selector_init (GnomeScanResolutionSelector *widget)
 
   priv->dispose_has_run = FALSE;
   priv->context		= NULL;
-  priv->x		= NULL;
-  priv->xscale		= NULL;
-  priv->xspin		= NULL;
-  priv->y		= NULL;
-  priv->yscale		= NULL;
-  priv->yspin		= NULL;
-  priv->chain		= NULL;
-  priv->chained		= FALSE;
+  priv->adjustment	= NULL;
+  priv->scale		= NULL;
+  priv->spin		= NULL;
+  priv->combo		= NULL;
 }
 
 void
@@ -233,6 +236,7 @@ gnome_scan_resolution_selector_new (GnomeScanContext *context)
   GtkWidget *widget, *button, *icon;
   GnomeScanResolutionSelector *gsrs;
   GnomeScanResolutionSelectorPrivate *priv;
+  GtkCellRenderer *renderer;	
 
   widget = GTK_WIDGET (g_object_new (GNOME_TYPE_SCAN_RESOLUTION_SELECTOR,
 				     "context", context,
@@ -243,40 +247,55 @@ gnome_scan_resolution_selector_new (GnomeScanContext *context)
   gtk_table_set_col_spacings (GTK_TABLE (widget), 6);
   gtk_table_set_row_spacings (GTK_TABLE (widget), 6);
 
-  /* X */
-  priv->xscale = gtk_hscale_new (NULL);
-  priv->x = gtk_range_get_adjustment (GTK_RANGE (priv->xscale));
-  gtk_scale_set_draw_value (GTK_SCALE (priv->xscale), FALSE);
-  gtk_range_set_update_policy (GTK_RANGE (priv->xscale), GTK_UPDATE_CONTINUOUS);
-  gtk_range_set_increments (GTK_RANGE (priv->xscale),
+  priv->scale = gtk_hscale_new (NULL);
+  priv->adjustment = gtk_range_get_adjustment (GTK_RANGE (priv->scale));
+  gtk_scale_set_draw_value (GTK_SCALE (priv->scale), FALSE);
+  gtk_range_set_update_policy (GTK_RANGE (priv->scale), GTK_UPDATE_CONTINUOUS);
+  gtk_range_set_increments (GTK_RANGE (priv->scale),
 			    5,	/* seems buggy, value is doubled */
 			    100);
 
-  priv->xspin = gtk_spin_button_new (priv->x, .5, 0);
-  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (priv->xspin), GTK_UPDATE_CONTINUOUS);
-  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (priv->xspin), TRUE);
+  priv->spin = gtk_spin_button_new (priv->adjustment, .5, 0);
+  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (priv->spin), GTK_UPDATE_CONTINUOUS);
+  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (priv->spin), TRUE);
+
+  priv->liststore = gtk_list_store_new (ENUM_N_COLUMNS,
+  					G_TYPE_STRING,
+					G_TYPE_VALUE);
+  priv->combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (priv->liststore));
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->combo), renderer, TRUE);
+  gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->combo), renderer,
+                                "text", ENUM_LABEL);
 
   gtk_table_attach (GTK_TABLE (widget),
-		    gtk_label_new (_("X resolution")),
+		    gtk_label_new (_("Resolution")),
 		    0, 1,
 		    0, 1,
 		    GTK_FILL,
 		    GTK_FILL,
 		    0, 0);
   gtk_table_attach (GTK_TABLE (widget),
-		    priv->xscale,
+		    priv->scale,
 		    1, 2,
 		    0, 1,
 		    GTK_EXPAND | GTK_FILL,
 		    GTK_FILL,
 		    0, 0);
   gtk_table_attach (GTK_TABLE (widget),
-		    priv->xspin,
+		    priv->spin,
 		    2, 3,
 		    0, 1,
 		    GTK_FILL,
 		    GTK_FILL,
 		    0, 0);
+  gtk_table_attach (GTK_TABLE (widget),
+                    priv->combo,
+                    2, 3,
+                    0, 1,
+                    GTK_FILL,
+                    GTK_FILL,
+                    0, 0);
   gtk_table_attach (GTK_TABLE (widget),
 		    gtk_label_new (_("dpi")),
 		    3, 4,
@@ -285,82 +304,16 @@ gnome_scan_resolution_selector_new (GnomeScanContext *context)
 		    GTK_FILL,
 		    0, 0);
 
-  g_signal_connect (GTK_RANGE (priv->xscale),
+  g_signal_connect (GTK_RANGE (priv->scale),
 		    "value-changed",
-		    G_CALLBACK (gsrs_value_changed),
+		    G_CALLBACK (gsrs_range_value_changed),
 		    gsrs);
 
-  /* Y */
-  priv->yscale = gtk_hscale_new (NULL);
-  priv->y = gtk_range_get_adjustment (GTK_RANGE (priv->yscale));
-  g_object_ref (priv->y);
-  gtk_scale_set_draw_value (GTK_SCALE (priv->yscale), FALSE);
-  gtk_range_set_update_policy (GTK_RANGE (priv->yscale), GTK_UPDATE_CONTINUOUS);
-  gtk_range_set_increments (GTK_RANGE (priv->yscale),
-			    5,	/* seems buggy, value is doubled */
-			    100);
+  g_signal_connect (GTK_COMBO_BOX (priv->combo),
+                    "changed",
+                    G_CALLBACK (gsrs_combo_value_changed),
+                    gsrs);
 
-  priv->yspin = gtk_spin_button_new (priv->y, .5, 0);
-  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (priv->yspin), GTK_UPDATE_CONTINUOUS);
-  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (priv->yspin), TRUE);
-
-  gtk_table_attach (GTK_TABLE (widget),
-		    gtk_label_new (_("Y resolution")),
-		    0, 1,
-		    1, 2,
-		    GTK_FILL,
-		    GTK_FILL,
-		    0, 0);
-  gtk_table_attach (GTK_TABLE (widget),
-		    priv->yscale,
-		    1, 2,
-		    1, 2,
-		    GTK_EXPAND | GTK_FILL,
-		    GTK_FILL,
-		    0, 0);
-  gtk_table_attach (GTK_TABLE (widget),
-		    priv->yspin,
-		    2, 3,
-		    1, 2,
-		    GTK_FILL,
-		    GTK_FILL,
-		    0, 0);
-  gtk_table_attach (GTK_TABLE (widget),
-		    gtk_label_new (_("dpi")),
-		    3, 4,
-		    1, 2,
-		    GTK_FILL,
-		    GTK_FILL,
-		    0, 0);
-
-  g_signal_connect (GTK_RANGE (priv->yscale),
-		    "value-changed",
-		    G_CALLBACK (gsrs_value_changed),
-		    gsrs);
-
-  /* chain */
-  priv->chain = gtk_button_new ();
-  gtk_button_set_relief (GTK_BUTTON (priv->chain),
-			 GTK_RELIEF_NONE);
-  g_signal_connect (priv->chain,
-		    "clicked",
-		    (GCallback) gsrs_chain_clicked,
-		    gsrs);
-
-  icon = gtk_image_new_from_stock (GS_STOCK_VCHAIN_BROKEN,
-				   -1);
-
-  gtk_container_add (GTK_CONTAINER (priv->chain),
-		     icon);
-
-  gtk_table_attach (GTK_TABLE (widget),
-		    priv->chain,
-		    4, 5,
-		    0, 2,
-		    0,
-		    0,
-		    0, 0);
- 
   g_signal_connect (context,
 		    "scanner-selected",
 		    G_CALLBACK (gsrs_scanner_selected),
@@ -380,25 +333,44 @@ gsrs_scanner_selected (GnomeScanContext *context,
 {
   GnomeScanResolutionSelectorPrivate *priv = GET_PRIVATE (gsrs);
   GnomeScanRange *range = gnome_scanner_get_resolution_range (scanner);
-  gdouble xresolution, yresolution;
+  GValueArray *enumeration =  gnome_scanner_get_resolution_enum (scanner);
+  gdouble resolution;
 
-  if (range) {
-    gtk_widget_set_sensitive (GTK_WIDGET (gsrs), TRUE);
-    gnome_scan_context_get_resolutions (context, &xresolution, &yresolution);
-    gtk_range_set_range (GTK_RANGE (priv->xscale), range->lower, range->upper);
-    gtk_range_set_value (GTK_RANGE (priv->xscale), xresolution);
+  if (range || enumeration) {
 
-    if (gnome_scanner_has_xy_resolution (scanner)) {
-      gtk_range_set_range (GTK_RANGE (priv->yscale), range->lower, range->upper);
-      gtk_range_set_value (GTK_RANGE (priv->yscale), yresolution);
-      gtk_widget_set_sensitive (priv->chain, TRUE);
+    if (range) {
+      gtk_widget_set_sensitive (GTK_WIDGET (gsrs), TRUE);
+      gtk_widget_hide (priv->combo);
+      gtk_widget_show (priv->scale);
+      gtk_widget_show (priv->spin);
+      gnome_scan_context_get_resolution (context, &resolution);
+      gtk_range_set_range (GTK_RANGE (priv->scale), range->lower, range->upper);
+      gtk_range_set_value (GTK_RANGE (priv->scale), resolution);
     }
-    else {
-      gtk_widget_set_sensitive (priv->chain, FALSE);
-      if (!priv->chained) {
-	gtk_button_clicked (GTK_BUTTON (priv->chain));
+
+    if (enumeration) {
+      gtk_widget_set_sensitive (GTK_WIDGET (gsrs), TRUE);
+      gtk_widget_hide (priv->scale);
+      gtk_widget_hide (priv->spin);
+      gtk_widget_show (priv->combo);
+      
+      GtkTreeIter iter;
+      GValue *label, *value;
+      gint i;
+
+      label = g_new0(GValue, 1);
+      g_value_init (label, G_TYPE_STRING);
+      for (i = 0; i < enumeration->n_values; i++) {
+        gtk_list_store_append (priv->liststore, &iter);
+        gtk_list_store_set (priv->liststore, &iter,
+                            ENUM_LABEL, g_strdup_printf("%.0f", g_value_get_double (enumeration->values+i)),
+                            ENUM_VALUE, (enumeration->values+i),
+                            -1);
       }
+
+      gtk_combo_box_set_active (GTK_COMBO_BOX (priv->combo), 0);
     }
+
   }
   else {
     gtk_widget_set_sensitive (GTK_WIDGET (gsrs), FALSE);
@@ -406,41 +378,27 @@ gsrs_scanner_selected (GnomeScanContext *context,
 }
 
 void
-gsrs_value_changed (GtkRange *range,
+gsrs_range_value_changed (GtkRange *range,
 		    GnomeScanResolutionSelector *gsrs)
 {
-  gnome_scan_context_set_resolutions (GET_PRIVATE (gsrs)->context,
-				      gtk_range_get_value (range),
-				      gtk_range_get_value (range));
+  gnome_scan_context_set_resolution (GET_PRIVATE (gsrs)->context,
+				     gtk_range_get_value (range));
 }
 
 void
-gsrs_chain_clicked (GtkButton *button,
-		    GnomeScanResolutionSelector *gsrs)
+gsrs_combo_value_changed (GtkComboBox *combo,
+                    GnomeScanResolutionSelector *gsrs)
 {
   GnomeScanResolutionSelectorPrivate *priv = GET_PRIVATE (gsrs);
-  static const gchar *stocks[] = { GS_STOCK_VCHAIN,
-				   GS_STOCK_VCHAIN_BROKEN };
+  GtkTreeIter iter;
+  GValue *value;
 
-  GtkWidget *icon;
-  priv->chained = !priv->chained;
-  gint stock_n = priv->chained ? 0 : 1;
-
-  icon = gtk_bin_get_child (GTK_BIN (button));
-  gtk_image_set_from_stock (GTK_IMAGE (icon),
-			    stocks[stock_n],
-			    -1);
-
-  if (priv->chained) {
-    gtk_range_set_adjustment (GTK_RANGE (priv->yscale), priv->x);
-    gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (priv->yspin), priv->x);
-  }
-  else {
-    gtk_adjustment_set_value (priv->y,
-			      gtk_adjustment_get_value (priv->x));
-    gtk_range_set_adjustment (GTK_RANGE (priv->yscale), priv->y);
-    gtk_spin_button_set_adjustment (GTK_SPIN_BUTTON (priv->yspin), priv->y);
-  }
+  gtk_combo_box_get_active_iter (GTK_COMBO_BOX (priv->combo), &iter);
+  gtk_tree_model_get (GTK_TREE_MODEL (priv->liststore), &iter,
+                                      ENUM_VALUE, &value,
+                                      -1);
+  gnome_scan_context_set_resolution (priv->context,
+				     g_value_get_double (value));
 }
 
 
